import { world, system, EntityComponentTypes, EquipmentSlot, EntityDamageCause } from "@minecraft/server";

function clamp(value, min, max) {
    return Math.max(Math.min(value, max), min);
}

function setHelmet(entity, itemStack) {
    entity?.getComponent(EntityComponentTypes.Equippable)?.setEquipment(EquipmentSlot.Head, itemStack)
}

function hasAdaptation(itemStack) {
    if (itemStack?.getLore().length !== 0) {
        if (itemStack?.getLore()[1].includes("Adaptation")) return true; else false;
    }else return false;
}

function adaptDamage(damage, max) {
    let damageValue = Math.floor(damage);
    if (damageValue >= max) return max; else return damageValue;
}

function adapt(entity, damage, chance) {
    let chanceValue = Math.floor(Math.random()*chance);
    if (chanceValue < 1) {
        const getHelmet = entity?.getComponent(EntityComponentTypes.Equippable)?.getEquipment(EquipmentSlot.Head);
        getHelmet.setLore(["", `§r§9+${adaptDamage(damage, 20)} Adaptation`])
        setHelmet(entity, getHelmet)
        entity?.playAnimation("animation.adapting_wheel.adapt")
        entity?.dimension.playSound("misc.adapt", entity.getHeadLocation())
        if (entity?.typeId == "minecraft:player") entity?.onScreenDisplay.setActionBar(`§9+${adaptDamage(damage, 20)} Adaptation`);
    }
}






//adapting wheel class
class AdaptingWheelClass {
    setAdapt(entity, damage, chance) {
        if (entity?.typeId == "minecraft:player") {
            const getHelmet = entity?.getComponent(EntityComponentTypes.Equippable)?.getEquipment(EquipmentSlot.Head);
            if (getHelmet?.typeId == "mahoraga:adapting_wheel") {
                if (hasAdaptation(getHelmet) == false) {
                    adapt(entity, damage, chance)
                }
            }
        }
    }
    adaptDefense(entity, damage, chance) {
        if (entity?.typeId == "minecraft:player") {
            const getHelmet = entity?.getComponent(EntityComponentTypes.Equippable)?.getEquipment(EquipmentSlot.Head);
            if (getHelmet?.typeId == "mahoraga:adapting_wheel") {
                if (hasAdaptation(getHelmet) == true) {
                    const getAdaptation = getHelmet?.getLore()[1].split(" ")[0].slice(getHelmet?.getLore()[1].split(" ")[0].indexOf("+")+1);
                    const health = entity?.getComponent(EntityComponentTypes.Health);
                    health.setCurrentValue(clamp(health.currentValue+clamp(damage, 0, getAdaptation), 0, 20))
                    if (damage > getAdaptation) {
                        adapt(entity, damage, chance)
                    }
                }
            }
        }
    }
    adaptOffense(entity, hurtEntity) {
        if (entity?.typeId == "minecraft:player") {
            const getHelmet = entity?.getComponent(EntityComponentTypes.Equippable)?.getEquipment(EquipmentSlot.Head);
            if (getHelmet?.typeId == "mahoraga:adapting_wheel") {
                if (hasAdaptation(getHelmet) == true) {
                    const getAdaptation = Number(getHelmet?.getLore()[1].split(" ")[0].slice(getHelmet?.getLore()[1].split(" ")[0].indexOf("+")+1));
                    hurtEntity?.getComponent(EntityComponentTypes.Health).setCurrentValue(clamp((hurtEntity?.getComponent(EntityComponentTypes.Health).currentValue-getAdaptation), 0, hurtEntity?.getComponent(EntityComponentTypes.Health).defaultValue))
                    hurtEntity?.dimension.spawnParticle("minecraft:critical_hit_emitter", {x: hurtEntity.location.x, y: hurtEntity.location.y+1.5, z: hurtEntity.location.z})
                }
            }
        }
    }
    adaptMax(entity) {
        const getHelmet = entity?.getComponent(EntityComponentTypes.Equippable)?.getEquipment(EquipmentSlot.Head);
        if (getHelmet?.typeId == "mahoraga:adapting_wheel") {
            if (hasAdaptation(getHelmet) == true) {
                const getAdaptation = getHelmet?.getLore()[1].split(" ")[0].slice(getHelmet?.getLore()[1].split(" ")[0].indexOf("+")+1);
                if (getAdaptation >= 20) {
                    entity?.addEffect("resistance", 10, {amplifier: 1, showParticles: false})
                }
            }
        }
    }
}
const adaptingWheel = new AdaptingWheelClass();






const adaptingWheelRunner = system.runInterval(() => {
    const players = world.getPlayers();
    for (let player of players) {
        adaptingWheel.adaptMax(player)
    }
})

world.afterEvents.entityHurt.subscribe((data) => {
    const { damage, hurtEntity } = data;
    const source = data.damageSource.damagingEntity;

    adaptingWheel.setAdapt(hurtEntity, damage, 3)
    adaptingWheel.adaptDefense(hurtEntity, damage, 3)
    adaptingWheel.adaptOffense(source, hurtEntity)
})